/*******************************************************************************

   Interruptor por nvel de tenso (pr-programvel) - verso 2 canais

   C/ PIC 12f675

   Autor: Cludio Lrios

   
   Para programar:
   Ajuste para uma tenso na entrada entre 0 e 5 volts que seja usada como
   um valor secreto de acionamento. Aperte o boto 'PROG'. Ir piscar o led
   de programao, para indicar a gravao do valor na EEprom interna.
   Uso:
   Toda vez que a tenso programada for colocada na entrada do equipamento, aps
   um tempo de 4 segundos, ir acionar a sada 1. 
   Alterando esta tenso e retornando ao valor programado, mantendo por 4 segundos
   ir acionar a saida 2, mantendo ligada a sada 1.
   Alterando a tenso novamente e retornando ao valor programado, mantendo por 4 
   segundos, ir desligar ambas as sadas. 
   O ciclo ser reiniciado.
   
   
   Use CCS C Compiler.

   Data: 23/04/2016

   Uso didtico

   Este arquivo  parte integrante do blog "LARIOS.TECNOLOGIA.WS'

*******************************************************************************/

#include <12F675.h>
#device  adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT

#use fast_io(A)  //  sentido das portas feita pelo programador

#rom 0x2102 = {0,0,0}
#byte   gpio = 0X05
#byte   tris = 0X85


#bit    prog = 0x5.1  //interruptor de programao pino 6
#bit    led  = 0x5.5  //pino 2
#bit    saida1= 0x5.4  //pino 3
#bit    saida2= 0x5.0  //pino 7

unsigned int16 valor,valeep;
unsigned int8  cnt,vez;
unsigned int1  f_t;

#define tolerance 20  //tolerancia permitida entre medies
#define vdel  100     //tempo que a tenso deve ficar constante
                      //antes de acionar  (100 x 40ms= 4 seg)
                      //para eliminar o tempo coloque apenas 1 ao invs de 100


/*******************************************************************************
           ROTINA MAIN
*******************************************************************************/

void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_adc_ports(sAN2 | VSS_VDD);  //seleciona AN2 e 5v como referncia
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   delay_us(10); //delay
   tris=0b001110; //direo das portas (tudo entrada,exceto pino 2 e 3 como sadas)
   gpio=0;
   cnt=vdel; //carga de cnt
   f_t=0;//destrava flag
   saida1=read_eeprom(2);//retorna valores aps reset
   saida2=read_eeprom(3);//idem
   vez=read_eeprom(4);//idem

/*******************************************************************************
           LAO PRINCIPAL DE REPETIO
*******************************************************************************/

  for(;;){
   if(!prog){//se foi apertado boto de programao do nivel de tenso
    valor = (int16)READ_ADC();// l valor da tenso de entrada do pino 5
    if(valor){//aceita valores maiores que zero apenas
    write_eeprom(0,valor>>8);//salva na eeprom interna valor high
    write_eeprom(1,valor);//salva na eeprom interna valor low
    led=1;//pisca led
    delay_ms(300);//delay 300 mseg
    led=0;
    }
    while(!prog)delay_ms(50);//aguarda soltar interruptor de programao
   }
   else{//modo uso
    delay_ms(40);//delay entre leituras do adc
    valor = (int16) READ_ADC();// l valor da tenso de entrada do pino 5
    valeep= (unsigned int16) read_eeprom(0)<<8|read_eeprom(1);//obtem o que foi gravado
    if(((valor+tolerance)>=valeep)&&(valor<=valeep+tolerance)){//compara dentro da tolerancia


      if((--cnt==0)&&!f_t){ //se tempo foi alcanado com a tenso correta
         if(vez==0){
          saida1=1;//aciona sada 1
          f_t=1;
          write_eeprom(2,1);//salva na eeprom
         }


         if(vez==1){
          saida2=1; //aciona sada 2
          f_t=1;
          write_eeprom(3,1);//salva na eeprom
         }


         if(vez==2){
         saida1=0; //desliga ambas as sadas
         saida2=0;
          f_t=1;
          write_eeprom(2,0);//salva na eeprom
          write_eeprom(3,0);//salva na eeprom
         }


         if(++vez>2) vez=0;//incrementa
         write_eeprom(4,vez);


      }

     
    }//if(((valor+tolerance)>=valeep)&&(valor<=valeep+tolerance))
    
    else{
        cnt=vdel; //carga de cnt
        f_t=0;//destrava flag
      }

    
    
   }//else
  }//for(;;)

}//void main()
